{% load i18n %}
{% load static %}
{% load load_helpdesk_settings %}
{% with request|load_helpdesk_settings as helpdesk_settings %}
<!DOCTYPE html>
<html lang="en">

<head>

    {% include 'helpdesk/base-head.html' %}
    {% block helpdesk_head %}{% endblock %}

</head>

<body id="bg-dark">

    {% include "helpdesk/navigation-header.html" %}

    <div id="wrapper">
        {% include "helpdesk/navigation-sidebar.html" %}

        <div id="content-wrapper">

            <div class="container-fluid">

                <!-- Breadcrumbs-->
                <ol class="breadcrumb">
                    {% block helpdesk_breadcrumb %}{% endblock %}
                </ol>

                {% block helpdesk_body %}{% endblock %}

            </div>
            <!-- /.container-fluid -->

            {% include "helpdesk/attribution.html" %}
        </div>
        <!-- /.content-wrapper -->

    </div>
    <!-- /#wrapper -->

    {% include "helpdesk/debug.html" %}

    {% include 'helpdesk/base_js.html' %}
    {% block helpdesk_js %}{% endblock %}
</body>
</html>
{% endwith %}
