{% load i18n %}

<div class="alert alert-danger alert-dismissible fade show" role="alert">
      {% trans "There are errors in the form" %}.
      <button type="button" class="close" data-dismiss="alert" aria-label="Close">
          <span aria-hidden="true">&times;</span>
      </button>
</div>