{% extends "helpdesk/public_base.html" %}
{% load i18n %}

{% block helpdesk_title %}{% blocktrans with category.title as kbcat %}{{ kbcat }}{% endblocktrans %}{% endblock %}

{% block helpdesk_breadcrumb %}
<li class="breadcrumb-item">
    <a href="{% url 'helpdesk:kb_index' %}">{% trans "Knowledgebase" %}</a>
</li>
<li class="breadcrumb-item active">{% blocktrans with category.title as kbcat %}{{ kbcat }}{% endblocktrans %}</li>
{% endblock %}

{% block helpdesk_body %}
{% include 'helpdesk/kb_category_base.html' %}
</div>

{% endblock %}
