{% load i18n humanize %}
{% load static %}
  <div class="form-row">
    <div class="col col-sm-3">
      <label for='id_sort' class="col-form-label">{% trans "Sorting" %}:</label>
    </div>
    <div class="col col-sm-3">
            <select id='id_sortx' name='sortx' class="">
                <option value='created'{% if query_params.sorting == "created"%} selected='selected'{% endif %}>
                    {% trans "Created" %}
                </option>
                <option value='title'{% if query_params.sorting == "title"%} selected='selected'{% endif %}>
                    {% trans "Title" %}
                </option>
                <option value='queue'{% if query_params.sorting == "queue"%} selected='selected'{% endif %}>
                    {% trans "Queue" %}
                </option>
                <option value='status'{% if query_params.sorting == "status"%} selected='selected'{% endif %}>
                    {% trans "Status" %}
                </option>
                <option value='priority'{% if query_params.sorting == "priority"%} selected='selected'{% endif %}>
                    {% trans "Priority" %}
                </option>
                <option value='assigned_to'{% if query_params.sorting == "assigned_to"%} selected='selected'{% endif %}>
                    {% trans "Owner" %}
                </option>
            </select>
  </div>
  <div class="col col-sm-2">
    <label for='id_sortreverse' class="col-form-label">{% trans "Reverse" %}</label>
  </div>
  <div class="col col-sm-3">
    <input type='checkbox' name='sortreverse' id='id_sortreverse'{% if query_params.sortreverse %} checked='checked'{% endif %} />
  </div>
  <div class="col col-sm-1">
    <button class="filterBuilderRemove btn btn-danger btn-sm float-right"><i class="fas fa-trash-alt"></i></button>
  </div>
   <div class="form-row filterHelp">{% trans "Ordering applied to tickets" %}</div>
</div>
