{% load i18n %}

<!-- Icon Cards-->
          <div class="row">
            {% for entry in basic_ticket_stats.open_ticket_stats %}
            <div class="col-xl-3 col-sm-6 mb-3">
              <div class="card text-white bg-{{ entry.2 }} o-hidden h-100">
                <div class="card-body">
                  <div class="card-body-icon">
                    <i class="fas fa-fw fa-list"></i>
                  </div>
                  <div class="mr-5">{{ entry.1 }} {{ entry.0 }}</div>
                </div>
                <a class="card-footer text-white clearfix small z-1" href="{% if entry.1 > 0 %}{% url 'helpdesk:list' %}?{{ entry.3 }}{% else %}#{% endif %}">
                  <span class="float-left">{% if entry.1 > 0 %}{% trans "View Tickets" %}{% else %}{% trans "No tickets in this range" %}{% endif %}</span>
                  <span class="float-right">
                    <i class="fas fa-angle-right"></i>
                  </span>
                </a>
              </div>
            </div>
            {% endfor %}
          </div>


