# Generated by Django 2.2.9 on 2020-01-27 16:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('helpdesk', '0028_kbitem_team'),
    ]

    operations = [
        migrations.AddField(
            model_name='kbcategory',
            name='public',
            field=models.BooleanField(default=True, verbose_name='Is KBCategory publicly visible?'),
        ),
    ]
